
from stm import tdict

class TObject(object):
    """
    An abstract class that causes all of its subclass's attributes to be backed
    by a TDict. This results in the subclass's attributes being transactional,
    so that they can be modified during a transaction without having to
    explicitly wrap all of them with TVars.
    """
    def __init__(self):
        object.__setattr__(self, "_tobject_dict", tdict.TDict())
    
    def __getattr__(self, name):
        return self._tobject_dict[name]
    
    def __setattr__(self, name, value):
        self._tobject_dict[name] = value
    
    def __delattr__(self, name):
        del self._tobject_dict[name]
    
    def __dir__(self):
        return self._tobject_dict.keys()






from stm.avl import Node, empty, balance, pop_leftmost
from collections import MutableMapping
import stm

# TODO: I'm pretty sure the first half or so of tdict and tlist can be merged
# together; I just need to figure out how to properly deal with the fact that
# tlist has to rewrite indexes when descending into right nodes. Perhaps have a
# function we pass to avl's hypothetical insert, delete, etc. functions that
# hand back whether we should go left, right, or whether we found a match or
# something, and also indicate a new "key" to use to recursively search. Needs
# some more thought.

def _dict_insert(node, key, value):
    if node is empty:
        # Empty (we never found a matching value, so the specified value isn't
        # already present), so we just return a new value containing the value
        # we were called with.
        return Node(empty, (key, value), empty)
    if key < node.value[0]: # Key is less than this node's key, so go left
        return balance(Node(_dict_insert(node.left, key, value), node.value, node.right))
    elif key > node.value[0]: # Key is greater than this node's key, so go right
        return balance(Node(node.left, node.value, _dict_insert(node.right, key, value)))
    else: # Key is equal to this node's key, so just replace this node's value
        return Node(node.left, (key, value), node.right)            


def _dict_delete(node, key):
    if node is empty:
        raise KeyError(key)
    if key < node.value[0]: # Go left
        return balance(Node(_dict_delete(node.left, key), node.value, node.right))
    elif key > node.value[0]: # Go right
        return balance(Node(node.left, node.value, _dict_delete(node.right, key)))
    else: # We're supposed to delete this node. The logic here is identical to
        # that of tlist.list_delete; see that function's documentation for
        # info and comments on how this one's works.
        if node.left is empty and node.right is empty:
            return empty
        elif node.left is empty:
            return node.right
        elif node.right is empty:
            return node.left
        else:
            new_value, new_right = pop_leftmost(node.right)
            return balance(Node(node.left, new_value, new_right))


def _dict_get(node, key):
    if node is empty:
        raise KeyError(key)
    if key < node.value[0]: # Go left
        return _dict_get(node.left, key)
    elif key > node.value[0]: # Go right
        return _dict_get(node.right, key)
    else: # Found the key; just return the value part of its tuple
        return node.value[1]


def _dict_iter(node, value_slice):
    if node is empty:
        return
    for child in _dict_iter(node.left, value_slice):
        yield child
    yield node.value[value_slice]
    for child in _dict_iter(node.right, value_slice):
        yield child


class TDict(MutableMapping):
    """
    A transactional dictionary.
    
    Internally, transactional dicts are maintained with a single TVar holding a
    copy-on-write binary tree annotated with dict keys. Insertion (and
    appending), removal, and lookup are therefore all O(log n) operations.
    len() is O(1), as is iter(), iterkeys(), iteritems(), and itervalues().
    
    One nice property of using a copy-on-write binary tree is iteration: the
    iterator returned from iter(tdict) is a snapshot of the dict's keys at that
    point in time. The dict can therefore be safely modified during iteration,
    without affecting the keys produced by the iteration. The same is, of
    course, true of iterkeys, iteritems, and itervalues.
    
    All of TDict's functions must be called within an STM transaction, with the
    exception of __str__/__repr__, which, for the sake of convenience,
    wrap themselves in a call to stm.atomically() internally. 
    """
    def __init__(self):
        self.var = stm.TVar(empty)
    
    def __getitem__(self, key):
        return _dict_get(self.var.get(), key)
    
    def __setitem__(self, key, value):
        self.var.set(_dict_insert(self.var.get(), key, value))
    
    def __delitem__(self, key):
        self.var.set(_dict_delete(self.var.get(), key))
    
    def __iter__(self):
        return _dict_iter(self.var.get(), 0)
    
    def __len__(self):
        return self.var.get().weight
    
    def iterkeys(self):
        return iter(self)
    
    def keys(self):
        return list(self.iterkeys())
    
    def itervalues(self):
        return _dict_iter(self.var.get(), 1)
    
    def values(self):
        return list(self.itervalues())
    
    def iteritems(self):
        return _dict_iter(self.var.get(), slice(None))
    
    def items(self):
        return list(self.iteritems())
    
    def __str__(self):
        return "TDict(%r)" % stm.atomically(lambda: dict(self))





















"""
A pure-Python software transactional memory system.

Have a look at the documentation for atomically() for more information.
"""

from threading import local as _Local, Lock as _Lock
from Queue import Queue, Full

class _RetryImmediately(BaseException):
    """
    Raised when a transaction needs to retry immediately. This happens when an
    attempt is made to read a variable that has been modified since the
    transaction started. It also happens just after the transaction has
    finished blocking in response to a _RetryLater.
    """
    pass

class _RetryLater(BaseException):
    """
    Raised when a transaction should retry at some later point, when at least
    one of the variables it accessed has been modified. This happens when
    retry() is called, and causes the toplevel transaction to block until one
    of the variables accessed in this transaction has been modified; the
    toplevel transaction then converts this into a _RetryImmediately.
    """
    pass


class _State(_Local):
    """
    A thread local holding the thread's current transaction
    """
    def __init__(self):
        self.current = None
    
    def get_current(self):
        """
        Returns the current transaction, or raises an exception if there is no
        current transaction.
        """
        if not self.current:
            raise Exception("No current transaction. The function you're "
                            "calling most likely needs to be wrapped in a "
                            "call to stm.atomically().")
        return self.current

_stm_state = _State()
# Lock that we lock on while committing transactions
_global_lock = _Lock()
# Number of the last transaction to successfully commit. The first transaction
# run will change this to the number 1, the second transaction to the number
# 2, and so on.
_last_transaction = 0


class _Transaction(object):
    """
    An abstract class for transactions that provides functionality to track the
    values of variables read and written during this transaction. The subclass
    is responsible for providing the values of variables which are not yet
    known and for running functions within the scope of this transaction.
    """
    def __init__(self):
        self.vars = {}
    
    def get_real_value(self, var):
        """
        Returns the real value of the specified variable, possibly throwing
        _RetryImmediately if the variable has been modified since this
        transaction started. This will only be called once for any given var in
        any given transaction; the value will thereafter be stored in
        self.vars. This must be overridden by the subclass.
        """
        raise NotImplementedError
    
    def run(self, function):
        """
        Runs the specified function in the context of this transaction,
        committing changes as needed once finished. This must be overridden by
        the subclass.
        """
        raise NotImplementedError
    
    def get_value(self, var):
        """
        Looks up the value of the specified variable in self.vars and returns
        it, or calls self.get_real_value(var) (and then stores it in self.vars)
        if the specified variable is not in self.vars. This is a concrete
        function; subclasses need not override it.
        """
        try:
            return self.vars[var]
        except KeyError:
            value = self.get_real_value(var)
            self.vars[var] = value
            return value
    
    def set_value(self, var, value):
        """
        Sets the entry in self.vars for the specified variable to the specified
        value.
        """
        self.vars[var] = value


class _BaseTransaction(_Transaction):
    """
    A toplevel transaction. This class takes care of committing values to the
    actual variables' values (and synchronizing on the global lock while doing
    so), blocking until vars are modified when a _RetryLater is caught, and so
    forth.
    """
    def __init__(self):
        _Transaction.__init__(self)
        self.parent = None
        # Store off the transaction id we're starting at, so that we know if
        # things have changed since we started.
        with _global_lock:
            self.start = _last_transaction
    
    def get_real_value(self, var):
        # Just check to make sure the variable hasn't been modified since we
        # started (and raise _RetryImmediately if it has), then return its real
        # value.
        with _global_lock:
            if var._modified > self.start:
                raise _RetryImmediately
            return var._real_value
    
    def run(self, function):
        global _last_transaction
        try:
            # First we actually run the transaction.
            result = function()
            # _Transaction appears to have run successfully. Now we need to make
            # sure nothing it used changed in the mean time.
            with _global_lock:
                for var in self.vars:
                    if var._modified > self.start:
                        # Var was modified since we started, so we need to
                        # retry against its new value.
                        raise _RetryImmediately
                # Nothing changed, so we're good to commit. First we make
                # ourselves a new id.
                _last_transaction += 1
                modified = _last_transaction
                # Then we update the real values of all of the TVars. Note that
                # TVar._update_real_value takes care of notifying the TVar's
                # queues for us.
                for var, value in self.vars.iteritems():
                    var._update_real_value(value, modified)
                # And we're done!
                return result
        except _RetryLater:
            # Received a retry request that made it all the way up to the top.
            # First, check to see if any of the variables we've accessed have
            # been modified since we started, which could change whether or not
            # we need to retry.
            with _global_lock:
                for var in self.vars:
                    if var._modified > self.start:
                        # Yep, one changed. Retry immediately.
                        raise _RetryImmediately
                # Nope, none of them have changed. So now we create a queue,
                # then add it to all of the vars we need to watch.
                q = Queue(1)
                for var in self.vars:
                    var._queues.add(q)
            # Then we wait.
            q.get()
            # One of the vars was modified. Now we go remove ourselves from
            # the vars' queues.
            with _global_lock:
                for var in self.vars:
                    var._queues.remove(q)
            # And then we retry immediately.
            raise _RetryImmediately


class _NestedTransaction(_Transaction):
    """
    A nested transaction. This just wraps another transaction and persists
    changes to it upon committing unless the function to run throws an
    exception (of any sort, including _RetryImmediately and _RetryLater).
    """
    def __init__(self, parent):
        _Transaction.__init__(self)
        self.parent = parent
    
    def get_real_value(self, var):
        # Just get the value from our parent.
        return self.parent.get_value(var)
    
    def run(self, function):
        # Run the function, then (if it didn't throw any exceptions;
        # _RetryImmediately, _RetryLater, or otherwise) copy our values into
        # our parent.
        result = function()
        for var, value in self.vars.iteritems():
            self.parent.set_value(var, value)
        return result


class TVar(object):
    """
    A transactional variable.
    
    TVars are the main primitives used within the STM system. They can only be
    read or written from within a call to atomically().
    
    More complex datatypes (such as TList, TDict, and TObject) are available in
    the tlist, tdict, and tobject modules, respectively.
    """
    __slots__ = ["_queues", "_real_value", "_modified"]
    
    def __init__(self, value=None):
        """
        Creates a TVar with the specified initial value.
        """
        self._queues = set()
        self._real_value = value
        self._modified = 0
    
    def get(self):
        """
        Returns the current value of this TVar.
        
        This can only be called from within a call to atomically(). An
        exception will be thrown if this method is called elsewhere.
        """
        # Ask the current transaction for our value.
        return _stm_state.get_current().get_value(self)
    
    def set(self, value):
        """
        Sets the value of this TVar to the specified value.
        
        This can only be called from within a call to atomically(). An
        exception will be thrown if this method is called elsewhere.
        """
        # Set the specified value into the current transaction.
        _stm_state.get_current().set_value(self, value)
    
    value = property(get, set)
    
    def _update_real_value(self, value, modified):
        # Update our real value and modified transaction
        self._real_value = value
        self._modified = modified
        # Then notify all of the queues registered to us.
        for q in self._queues:
            try:
                q.put(None, False)
            except Full:
                pass 


def atomically(function):
    """
    Run the specified function in an STM transaction.
    
    Changes made to TVars from within a transaction will not be visible to
    other transactions until the transaction commits, and changes from other
    transactions started after this one started will not be seen by this one.
    The net effect is one of wrapping every transaction with a global lock, but
    without the loss of parallelism that would result.
    
    If the specified function throws an exception, the exception will be
    propagated out, and all of the changes made to TVars during the course of
    the transaction will be reverted.
    
    If a transaction detects state that it cannot yet continue with, it can
    call retry(). Conceptually, one could think of retry() as causing the
    transaction to immediately abort and restart from the top. However, the
    actual implementation causes the transaction to block until at least one of
    the variables it accessed has been modified; it then immediately restarts.
    
    One can use retry() to obtain blocking behavior by doing something like
    this:
    
        if resources_available.get() > 0:
            resources_available.set(resources_available.get() - 1)
        else:
            retry()
    
    This will block until there is at least one resource available, and then
    atomically decrement the number of resources remaining.
    
    Note that inconsistent state (i.e. state produced when a transaction has
    only run half-way to completion) will never be seen by any other
    transaction, so such situations do not need to be accounted for in retry()
    logic.
    
    atomically() fully supports nested transactions. If a nested transaction
    throws an exception, the changes it made are reverted, and the exception
    propagated out of the call to atomically().
    
    Two alternative transactions, both of which may retry(), can be combined
    together such that the first to succeed (i.e. not retry) is executed and
    the value it produces returned with or_else(). This can be used to obtain
    behavior similar to the Unix system call select(). For example, to read a
    value from either q1 or q2, whichever produces a value first, and block
    until either one has a value:
    
        value = or_else(q1.get, q2.get)
    
    or_else can also be used to make non-blocking variants of blocking
    functions. For example, given a queue with a blocking get() function, we
    can get the queue's value or, if it does not currently have a value
    available, return None with:
    
        or_else(queue.get, lambda: None)
    
    If all of the transactions passed to or_else retry, or_else itself retries.
    
    The return value of atomically() is the return value of the function that
    was passed to it.
    """
    while True:
        # If we have no current transaction, create a _BaseTransaction.
        # Otherwise, create a _NestedTransaction with the current one as its
        # parent.
        if _stm_state.current:
            transaction = _NestedTransaction(_stm_state.current)
        else:
            transaction = _BaseTransaction()
        # Then set it as the current transaction
        _stm_state.current = transaction
        # Then run the transaction. _BaseTransaction's implementation takes care
        # of catching _RetryLater and blocking until one of the vars we read is
        # modified, then converting it into a _RetryImmediately exception.
        try:
            return transaction.run(function)
        except _RetryImmediately:
            # We were asked to retry immediately. If we're a _BaseTransaction,
            # just continue. If we're a _NestedTransaction, propagate the
            # exception up. TODO: Figure out a way to move this logic into
            # individual methods on _Transaction that _BaseTransaction and
            # _NestedTransaction can override accordingly.
            if isinstance(transaction, _BaseTransaction):
                continue
            else:
                raise
        finally:
            # Before we go, restore the current transaction to whatever it was
            # before we started.
            _stm_state.current = transaction.parent


def retry():
    """
    Retries the current transaction. See the documentation for atomically()
    for more information.
    """
    raise _RetryLater


def or_else(*args):
    """
    Runs (and returns the value produced by) the first function passed to this
    function that does not retry, or retries if all of the passed-in functions
    retry (or if no arguments are passed in). See the documentation for
    atomically() for more information.
    
    Note that each function passed in is automatically run in its own nested
    transaction, so that the effects of those that end up retrying are reverted
    before the next function is run.
    """
    for function in args:
        # Try to run each function in sequence, in its own transaction so that
        # if it raises _RetryLater (or any other exception) its effects will be
        # undone.
        try:
            return atomically(function)
        except _RetryLater:
            # Requested a retry, so move on to the next alternative
            pass
    # All of the alternatives retried, so retry ourselves.
    retry()
















































import stm
from collections import MutableSet

class TSet(MutableSet):
    """
    A transactional mutable set.
    
    Right now, this actually just uses an underlying Python set which it copies
    on every write, so it's woefully inefficient. I'll be changing it to use
    the AVL trees that TList and TDict use soon, but I need to figure out how
    to properly implement weak sets in terms of them first. (I could always
    just have weak nodes pointing to dereferenced objects collected only during
    iteration, but then the iterators can't be completely isolated from the set
    itself, so that needs some thought.)
    """
    def __init__(self, initial_items=set(), backing_type=set):
        self.backing_type = backing_type
        self._var = stm.TVar(backing_type(initial_items))
    
    def __contains__(self, item):
        return self._var.get().__contains__(item)
    
    def __iter__(self):
        return self._var.get().__iter__()
    
    def __len__(self):
        return self._var.get().__len__()
    
    def add(self, item):
        new_set = self.backing_type(self._var.get())
        new_set.add(item)
        self._var.set(new_set)
    
    def discard(self, item):
        new_set = self.backing_type(self._var.get())
        new_set.discard(item)
        self._var.set(new_set)








"""
A simple module to draw graphs of AVL trees to png files.

Just call visualize(some_node, "some_file.png") to draw some_node (which should
be an avl.Node, or avl.empty)'s hierarchical representation to some_file.png.
Quite useful for debugging tree-related issues.
"""

import pydot
from stm_system import avl
from afn.utils.concurrent import AtomicInteger

next_id = AtomicInteger(1).get_and_add

def visualize(node, png_path):
    graph = pydot.Dot(graph_type="graph")
    draw_node(graph, node)
    graph.write_png(png_path)

def draw_node(graph, node):
    node_id = str(next_id())
    if node is avl.empty:
        graph.add_node(pydot.Node(node_id, label="<empty>", fontsize="9"))
    else:
        graph.add_node(pydot.Node(node_id, label='"' + repr(node.value) + 
                                  "\\nbalance: %s\\nheight: %s\\nweight: %s\"" %
                                  (node.balance, node.height, node.weight)))
        graph.add_edge(pydot.Edge(node_id, draw_node(graph, node.left)))
        graph.add_edge(pydot.Edge(node_id, draw_node(graph, node.right)))
    return node_id
        



"""
An AVL tree implementation internally used by TList and TDict.

This module provides an implementation of the AVL self-balancing binary tree
algorithm. TList and TDict use the trees provided by this module to store their
data.

At some point I hope to clean up the interface to this module a bit and then
make it a public module in its own right. For now, though, the interface is a
bit rough and some of the tree logic is in TList and TDict, but it's still
worth having a look around if you're curious about how AVL works.

Note that the version implemented here is copy-on-write. That is, once a node
is created, it is never modified; new nodes are created instead, pointing
(where appropriate) to old ones. The main reason for this is that TList and
TDict, by virtue of being transactional, need a copy-on-write data structure to
avoid having to inject TVars at every level of the tree, but interestingly
enough, a copy-on-write implementation of AVL turns out to be significantly
simpler to implement than an implementation that modifies the tree in place.

The core of the AVL algorithm is implemented in the balance() function. It
takes a node that may be out of balance by one level and returns a balanced
version of that node.
"""

from collections import MutableSequence

class _Empty(object):
    """
    Class of empty nodes. There is a singleton instance of this class, empty,
    that should be used; this class should never be instantiated separately.
    """
    def __init__(self):
        self.height = 0
        self.balance = 0
        self.weight = 0
    def __str__(self):
        return "empty"
    __repr__ = __str__
empty = _Empty()


class Node(object):
    """
    Class of nodes. Each node has a left child node, a value, and a right child
    node. Nodes also automatically compute their height (number of nodes to
    their farthest descendant, including themselves), balance (difference in
    height between the node's left and right subtrees), and weight (total
    number of nodes present in the subtree rooted at this node).
    """
    __slots__ = ["left", "value", "right", "height", "balance", "weight"]
    """
    Creates a new node with the specified left child (which may be either
    another Node instance or empty), value, and right child (which must obey
    the same constraints as the left child).
    """
    def __init__(self, left, value, right):
        self.left = left
        self.value = value
        self.right = right
        self.height = max(left.height, right.height) + 1
        # Positive numbers indicate left-heavy trees, negative numbers indicate
        # right-heavy trees
        self.balance = self.left.height - self.right.height
        self.weight = self.left.weight + 1 + self.right.weight
    
    def __str__(self):
        return "Node(%r, %r, %r)" % (self.left, self.value, self.right)
    
    __repr__ = __str__


def rotate_left(node):
    """
    Returns a node representing the left rotation of the specified node.
    """
    a = node.left
    b = node.right.left
    c = node.right.right
    return Node(Node(a, node.value, b), node.right.value, c)


def rotate_right(node):
    """
    Returns a node representing the right rotation of the specified node.
    """
    a = node.left.left
    b = node.left.right
    c = node.right
    return Node(a, node.left.value, Node(b, node.value, c))


def pop_leftmost(node):
    """
    Finds the leftmost descendant of the specified node (the first node that
    would be encountered, were a proper binary tree iteration to be performed
    on the specified node) and removes it, returning a tuple of the value of
    the node removed and the new node representing the tree with the leftmost
    node removed. If empty is passed in, an exception will be thrown.
    """
    if node.left is empty: # No left node, so this is the one we want. Return
        # its value, and replace it with its right node.
        return node.value, node.right
    else: # We can still keep walking left. Do so, but remember to balance on
        # our way back up.
        value, new_left = pop_leftmost(node.left)
        return value, balance(Node(new_left, node.value, node.right))


def balance(node):
    """
    Balances the specified node according to the rules of AVL.
    """
    # If the node is only off by one level, we just leave it be.
    if node.balance >= -1 and node.balance <= 1:
        return node
    if node.balance == -2:
        # The right child is two levels deeper than the left child, so we need 
        # to rotate left to get some of the weight moved to the left side. But 
        # first we see if the right child is left-heavy to avoid shifting too
        # much weight to the left side.
        if node.right.balance == 1:
            # Yep, it's left-heavy. Rotate it right to fix that.
            node = Node(node.left, node.value, rotate_right(node.right))
        # Then rotate ourselves left to shift the weight, and we're done.
        return rotate_left(node)
    elif node.balance == 2:
        # The left child is two levels deeper than the right child, so we
        # follow the same rules as for the opposite case, but with left and
        # right flipped.
        if node.left.balance == -1:
            node = Node(rotate_left(node.left), node.value, node.right)
        return rotate_right(node)
    else:
        # Tree's out of balance by more than two levels, which shouldn't ever
        # happen if we've implemented AVL correctly.
        raise Exception("Balance factor off (this is a bug in the AVL "
                        "tree implementation): %r for node %r" %
                        (node.balance, node))







from stm.avl import Node, empty as _empty, balance, pop_leftmost
from collections import MutableSequence
import stm

def _list_insert(node, index, value):
    if node is _empty:
        # Empty, so just return a node with our value. TODO: Could check that
        # the index is == 0; if it's greater, then the original insertion index
        # is greater than the length of the list plus one, which doesn't make
        # sense.
        return Node(_empty, value, _empty)
    if index < node.left.weight: # Go left
        return balance(Node(_list_insert(node.left, index, value), node.value, node.right))
    elif index > node.left.weight: # Go right, but subtract left.weight + 1
        # from the index
        return balance(Node(node.left, node.value, _list_insert(node.right, index - 1 - node.left.weight, value)))
    else: # Node's supposed to be inserted here, so insert it into our left
        # child as the node at its weight'th index, or its last node
        return balance(Node(_list_insert(node.left, node.left.weight, value), node.value, node.right))            


def _list_replace(node, index, value):
    if node is _empty: # Walked right off the right end of the tree, so the
        # index is too high (or walked off the left end of the tree if the
        # index was less than 0)
        raise IndexError
    if index < node.left.weight: # Index is less than the number of children we
        # have on our left, so the item we're looking for is on our left side
        return Node(_list_replace(node.left, index, value), node.value, node.right)
    elif index > node.left.weight: # Same, but it's on our right side. Note
        # that we subtract the number of items on the left (that we're skipping)
        # as well as 1 for ourselves from the index before passing it down.
        return Node(node.left, node.value, _list_replace(node.right, index - node.left.weight - 1, value))
    else: # We're the node they're looking for
        return Node(node.left, value, node.right)


def _list_delete(node, index):
    if node is _empty:
        raise IndexError
    if index < node.left.weight: # Go left
        return balance(Node(_list_delete(node.left, index), node.value, node.right))
    elif index > node.left.weight: # Go right
        return balance(Node(node.left, node.value, _list_delete(node.right, index - node.left.weight - 1)))
    else: # We're supposed to delete this node
        if node.left is _empty and node.right is _empty:
            # No children; just return empty
            return _empty
        elif node.left is _empty:
            # Only a right child; return it
            return node.right
        elif node.right is _empty:
            # Only a left child; return it
            return node.left
        else:
            # Both a left and a right child, slightly more tricky. What we do
            # is pop the next item in the list and replace ourselves with it.
            new_value, new_right = pop_leftmost(node.right)
            return balance(Node(node.left, new_value, new_right))


def _list_get(node, index):
    if node is _empty:
        raise IndexError
    if index < node.left.weight: # Go left
        return _list_get(node.left, index)
    elif index > node.left.weight: # Go right
        return _list_get(node.right, index - node.left.weight - 1)
    else: # Found the node, so return its value
        return node.value


def _list_iter(node):
    if node is _empty:
        return
    for child in _list_iter(node.left):
        yield child
    yield node.value
    for child in _list_iter(node.right):
        yield child


class TList(MutableSequence):
    """
    A transactional list.
    
    Internally, transactional lists are maintained with a single TVar holding a
    copy-on-write binary tree annotated with list indexes. Insertion (and
    appending), removal, and lookup are therefore all O(log n) operations.
    len() is O(1), as is iter().
    
    One nice property of using a copy-on-write binary tree is iteration: the
    iterator returned from iter(tlist) is a snapshot of the list at that point
    in time. The list can therefore be safely modified during iteration,
    without affecting the items produced by the iteration.
    
    All of TList's functions must be called within an STM transaction, with the
    exception of __str__/__repr__, which, for the sake of convenience,
    wrap themselves in a call to stm.atomically() internally. 
    """
    def __init__(self, initial_values=[]):
        self.var = stm.TVar(_empty)
        if initial_values:
            # TODO: This could probably be made to be O(n) instead of
            # O(n log n) like it is right now
            self.extend(initial_values)
    
    def __getitem__(self, index):
        return _list_get(self.var.get(), index)
    
    def __setitem__(self, index, value):
        self.var.set(_list_replace(self.var.get(), index, value))
    
    def __delitem__(self, index):
        self.var.set(_list_delete(self.var.get(), index))
    
    def __len__(self):
        return self.var.get().weight
    
    def insert(self, index, value):
        self.var.set(_list_insert(self.var.get(), index, value))
    
    def __str__(self):
        return "TList(%r)" % stm.atomically(lambda: list(self))
    
    def __iter__(self):
        return _list_iter(self.var.get())
    
    __repr__ = __str__





"""
A queue with multiple endpoints from which items can be consumed.

See the documentation for the BroadcastQueue class for more information.
"""

import stm
from stm import tobject

class Empty(Exception):
    """
    Exception thrown from endpoints' get() functions when block=False is passed
    in and no items are currently available.
    """
    pass


class _Item(object):
    """
    An item that has been added to the queue. Items themselves are immutable,
    but they contain a TVar pointing to the next item added to the queue, or
    None if a "next item" hasn't been added yet.
    """
    def __init__(self, value):
        self.value = value
        self.next = stm.TVar()


class BroadcastQueue(tobject.TObject):
    """
    A queue that can have multiple endpoints from which items can be consumed.
    
    Endpoints are created by calling new_endpoint(). Each endpoint initially
    starts out empty; items become available as soon as the creating queue's
    put() function is called.
    
    Items inserted into the queue with put() become available on all endpoints
    to consume. This allows BroadcastQueues to be used to broadcast values to
    several different consumers.
    
    When an endpoint is no longer needed, it can be simply discarded. Endpoints
    hold a reference to the queue they were created from, not the other way
    around, so they will be immediately garbage collected and any items unread
    by the endpoint but not by any other endpoint immediately reclaimed.
    
    An interesting side effect of this is that adding items to a queue that has
    never had any endpoints created from it, or one that has had all of its
    endpoints discarded, silently discards the items added to it, and is thus
    a no-op.
    """
    def __init__(self):
        tobject.TObject.__init__(self)
        self._var = stm.TVar(None)
    
    def put(self, value):
        """
        Inserts an item into this queue. The item will then become available on
        all endpoints created from it.
        """
        item = _Item(value)
        self._var.set(item)
        self._var = item.next
    
    def new_endpoint(self):
        """
        Creates a new endpoint that receives values added to this queue. The
        endpoint initially starts out empty; items will appear on it as soon as
        put() is called next.
        """
        return BroadcastEndpoint(self._var)


class BroadcastEndpoint(tobject.TObject):
    """
    A broadcast endpoint from which items can be read.
    
    Normally, you won't need to create instances of this class yourself;
    they're typically obtained by calling new_endpoint on a BroadcastQueue
    instance.
    """
    def __init__(self, var):
        tobject.TObject.__init__(self)
        self._var = var
    
    def get(self, block=True):
        """
        Removes and returns the next available item from this endpoint.
        
        If block is True and there aren't any items currently available on this
        endpoint, this function retries. If block is False, the Empty exception
        is raised instead.
        """
        if self._var.get() is None:
            if block:
                stm.retry()
            else:
                raise Empty
        else:
            item = self._var.get()
            self._var = item.next
            return item.value
        



"""
A prototype rewrite of the bindings system to use STM. This will likely be
moved into a separate package at some point in the near future.
"""

from collections import namedtuple
import collections
import weakref
from stm.tlist import TList
from stm.tdict import TDict
from stm.tset import TSet
from stm.tobject import TObject
import stm

SetValue = namedtuple("SetValue", ["value"])
# Change that indicates that our circuit is becoming synthetic. SetValue will
# be issued when it becomes concrete again.
LostValue = namedtuple("LostValue", [])

ModifyKey = namedtuple("ModifyKey", ["key", "value"])
DeleteKey = namedtuple("DeleteKey", ["key"])

InsertItem = namedtuple("InsertItem", ["index", "item"])
ReplaceItem = namedtuple("ReplaceItem", ["index", "item"])
DeleteItem = namedtuple("DeleteItem", ["index"])

NoValue = namedtuple("NoValue", [])

class SyntheticError(Exception):
    pass


class IdHash(object):
    __slots__ = ["value"]
    
    def __init__(self, value):
        self.value = value
    
    def __hash__(self):
        return id(self.value)
    
    def __cmp__(self, other):
        if not isinstance(other, IdHash):
            return NotImplemented
        return cmp(self.__hash__(), other.__hash__())


class StrongWeakSet(collections.MutableSet):
    def __init__(self):
        self._weak = TSet(backing_type=weakref.WeakSet)
        self._strong = TSet(backing_type=set)
    
    def add(self, item, weak=False):
        # This is thread-safe; since we have a strong reference to the item,
        # it won't just suddenly disappear from _weak if it's in there
        if item in self._weak or item in self._strong:
            pass
        if weak:
            self._weak.add(item)
        else:
            self._strong.add(item)
    
    def is_weak(self, item):
        return item in self._weak
    
    def discard(self, item):
        self._weak.discard(item)
        self._strong.discard(item)
    
    def __len__(self):
        return len(self._weak) + len(self._strong)
    
    def __iter__(self):
        for w in self._weak:
            yield w
        for s in self._strong:
            yield s
    
    def __contains__(self, item):
        return item in self._strong or item in self._weak
    
    def __str__(self):
        return "<StrongWeakSet: strong: %r, weak: %r>" % (self._strong, self._weak)
    
    __repr__ = __str__


# Probably not needed anymore, since copies of TDicts are O(1). TODO: Actually
# implement TDict.copy() to just create a copy with the same current root node.
class SimpleMapping(collections.Mapping):
    def __init__(self, get_function, iter_function):
        self.get_function = get_function
        self.iter_function = iter_function
    
    def __getitem__(self, key):
        return self.get_function(key)
    
    def __len__(self):
        i = 0
        for _ in self.iter_function():
            i += 1
        return i
    
    def __iter__(self):
        return self.iter_function()
    
    def __contains__(self, key):
        for k in self.iter_function():
            if k == key:
                return True
        return False


# Same note as SimpleMapping about not being needed
class SimpleSequence(collections.Sequence):
    def __init__(self, get_function, len_function):
        self.get_function
        self.len_function
    
    def __len__(self):
        return self.len_function()
    
    def __getitem__(self, index):
        return self.get_function(index)


class Bindable(object):
    def perform_change(self, change):
        raise NotImplementedError
    
    def get_value(self):
        raise NotImplementedError
    
    @property
    def binder(self):
        # TODO: Make sure we don't need to subclass TObject for this to work
        try:
            return self._binder
        except AttributeError:
            self._binder = Binder(self)
            return self._binder
    
    @property
    def is_synthetic(self):
        try:
            self.get_value()
            return False
        except SyntheticError:
            return True

# BIG NOTE ABOUT WEAK REFERENCES: Do /not/ weakly bind to a concrete binder
# from a synthetic one. Or in other words, if a binder that's weakly bound to
# is the last concrete binder in its circuit and it goes away, the rest of the
# circuit will /not/ be notified that it's now synthetic (there are far too
# many threading issues involved in doing this), so make sure that there's at
# least one concrete binder on the strong side of the circuit, or that both
# sides are synthetic. I might consider having warnings be printed out if the
# situation is detected, but they still won't be 100% reliable due to threading
# issues.

class Binder(object):
    def __init__(self, bindable):
        self.binders = StrongWeakSet()
        self.bindable = bindable
    
    def get_binders(self, binders=None):
        if binders is None:
            binders = TSet()
        if self not in binders:
            binders.add(self)
            for b in self.binders:
                b.get_binders(binders)
        return binders
    
    def get_value(self):
        for binder in self.get_binders():
            try:
                return binder.bindable.get_value()
            except SyntheticError:
                pass
        raise SyntheticError
    
    @property
    def is_synthetic(self):
        try:
            self.get_value()
            return False
        except SyntheticError:
            return True
        
    def bind(self, other, self_weak=False, other_weak=False):
        @stm.atomically
        def _():
            if other in self.binders: # Already bound, throw an exception
                raise Exception("Already bound")
            # Figure out whose value to keep; it's basically other's if we're
            # synthetic but other isn't, and self's otherwise
            if self.is_synthetic and not other.is_synthetic:
                keep, update = other, self
            else:
                keep, update = self, other
            update_binders = update.get_binders()
            if not keep.is_synthetic:
                # We get the value before linking to make sure we get our own
                # value and not the value of the binder we linked to
                keep_value = keep.get_value()
            # Link and add a reversion step that unlinks
            self.binders.add(other, other_weak)
            other.binders.add(self, self_weak)
            # If keep's synthetic, then other must be as well, so we're done.
            # If not, though, then we need to pass the new value to all of
            # update's former binders.
            if not keep.is_synthetic:
                for binder in update_binders:
                    if not binder.bindable.is_synthetic:
                        binder.bindable.perform_change(SetValue(keep_value))
                for binder in update_binders:
                    if binder.bindable.is_synthetic:
                        binder.bindable.perform_change(SetValue(keep_value))
            # That should be it.
    
    def unbind(self, other):
        @stm.atomically
        def _():
            if other not in self.binders: # Not bound, throw an exception
                raise Exception("Not bound")
            self_weak, other_weak = other.binders.is_weak(self), self.binders.is_weak(other)
            self.binders.discard(other)
            other.binders.discard(self)
            if (self.is_synthetic and not other.is_synthetic) or (other.is_synthetic and not self.is_synthetic):
                # If they're both synthetic then we don't need to do anything
                # else as we were already synthetic before we unbound. If
                # they're both concrete then both of them will keep their
                # values so we don't need to do anything. But if one is newly
                # synthetic, then we need to let it know that it's now
                # synthetic.
                synthetic = self if self.is_synthetic else other
                for binder in synthetic.get_binders():
                    binder.bindable.perform_change(LostValue())

    def notify_change(self, change, to_self=False):
        @stm.atomically
        def _():
            if isinstance(change, LostValue) and not self.is_synthetic:
                # We're now synthetic but another binder on our circuit is still
                # concrete, so don't do anything
                return
            for binder in self.get_binders():
                if (to_self or binder != self) and not binder.bindable.is_synthetic:
                    binder.bindable.perform_change(change)
            for binder in self.get_binders():
                if (to_self or binder != self) and binder.bindable.is_synthetic:
                    binder.bindable.perform_change(change)
    
    def perform_change(self, change):
        self.notify_change(change, to_self=True)


def bind(s, m, s_weak=True, m_weak=False):
    m.binder.bind(s.binder, m_weak, s_weak)

def s_bind_s(s, m):
    bind(s, m, False, False)

def s_bind_w(s, m):
    bind(s, m, False, True)

def w_bind_s(s, m):
    bind(s, m, True, False)

def w_bind_w(s, m):
    bind(s, m, True, True)


def unbind(a, b):
    a.binder.unbind(b.binder)


class SyntheticBindable(Bindable):
    # Bindable that can take on any sort of type, and exists mainly to serve
    # as a placeholder to bind other things to
    def get_value(self):
        raise SyntheticError
    
    def perform_change(self, change):
        pass


def value_str(value_bindable):
    @stm.atomically
    def _():
        try:
            return "<%s: %r>" % (type(value_bindable).__name__, value_bindable.binder.get_value())
        except SyntheticError:
            return "<%s: synthetic>" % type(value_bindable).__name__


def list_str(list_bindable):
    @stm.atomically
    def _():
        try:
            return "<%s: %r>" % (type(list_bindable).__name__, list(list_bindable.binder.get_value()))
        except SyntheticError:
            return "<%s: synthetic>" % type(list_bindable).__name__


def dict_str(dict_bindable):
    @stm.atomically
    def _():
        try:
            return "<%s: %r>" % (type(dict_bindable).__name__, dict(dict_bindable.binder.get_value()))
        except SyntheticError:
            return "<%s: synthetic>" % type(dict_bindable).__name__


class PyValueMixin(Bindable):
    # Mixin that exposes a Python-friendly interface for value bindables, in
    # the form of get and set functions and a value property
    def get(self):
        return self.binder.get_value()
    
    def set(self, value):
        self.binder.perform_change(SetValue(value))
    
    value = property(get, set)
    
    __str__ = value_str
    __repr__ = value_str


class MemoryValue(PyValueMixin, Bindable):
    # Concrete value that stores its value in memory
    def __init__(self, value):
        self._var = stm.TVar(value)
    
    def get_value(self):
        return self._var.get()
    
    def perform_change(self, change):
        # We're concrete, so we'll never see a LostValue change
        if not isinstance(change, SetValue):
            raise TypeError("Need a SetValue instance")
        self._var.set(change.value)


class PyValue(SyntheticBindable, PyValueMixin):
    # Synthetic value that allows the convenient interface of PyValueMixin to
    # be used with anything that can be bound to it
    def __init__(self, bindable):
        if bindable is not None:
            w_bind_s(self, bindable)


class PyDictMixin(Bindable, collections.MutableMapping):
    def __getitem__(self, key):
        return self.binder.get_value()[key]
    
    def __setitem__(self, key, value):
        self.binder.perform_change(ModifyKey(key, value))
    
    def __delitem__(self, key):
        self.binder.perform_change(DeleteKey(key))
    
    def __len__(self):
        return len(self.binder.get_value())
    
    def __iter__(self):
        return self.binder.get_value().__iter__()
    
    def __contains__(self, key):
        return self.binder.get_value().__contains__(key)
    
    __str__ = dict_str
    __repr__ = dict_str


class MemoryDict(Bindable):
    # Dictionary bindable that stores things in memory
    def __init__(self):
        self._dict = TDict()
    
    def get_value(self):
        # TODO: return self._dict.copy(), which will be O(1) once I properly
        # implement it
        return self._dict
    
    def perform_change(self, change):
        if isinstance(change, ModifyKey):
            self._dict[change.key] = change.value
        elif isinstance(change, DeleteKey):
            if change.key in self._dict:
                del self._dict[change.key]
            else: # Delete a non-existent key; throw an exception. Not sure at
                # the moment if we should just ignore it instead... Main
                # advantage to throwing a proper exception is that then
                # PyDict.__del__ correctly throws an exception when trying to
                # delete a key that doesn't actually exist.
                # UPDATE: Ignoring for now, as this causes problems with
                # DictController (setting the value to the delete-key sentinel
                # when it's already set to the sentinel causes a duplicate
                # DeleteKey to be issued to whatever's bound to the
                # controller's dict). But I've yet to see anything actually
                # relying on __del__ throwing an exception, so this should be
                # fine. And I'll revisit this if it becomes a problem later.
                pass
        elif isinstance(change, SetValue):
            # TODO: Might want to have TDict.update check to see if the other
            # dictionary is also a TDict and, if so, just copy its node, and if
            # not, invoke the superclass update implementation
            self._dict.clear()
            self._dict.update(change.value)
        else:
            raise TypeError("Need a ModifyKey or DeleteKey")


class PyDict(MemoryDict, PyDictMixin):
    # Synthetic bindable that exposes a collections.MutableMapping-compatible
    # interface for any other dictionary bindable
    def __init__(self, bindable=None):
        MemoryDict.__init__(self)
        if bindable is not None:
            w_bind_s(self, bindable)


class AttributeDict(MemoryDict):
    def __getattr__(self, key):
        try:
            return self.get_value()[key]
        except KeyError as e:
            raise AttributeError(e.args[0])
    
    def __setattr__(self, key, value):
        if key == "_dict" or key == "_binder":
            # Used by superclasses
            object.__setattr__(self, key, value)
        else:
            self.binder.perform_change(ModifyKey(key, value))
    
    def __delattr__(self, key):
        self.binder.perform_change(DeleteKey(key))
    
    def __dir__(self):
        return self.get_value().items()


class EmptyDict(Bindable):
    def get_value(self):
        return {}
    
    def perform_change(self, change):
        if isinstance(change, SetValue) and len(change.value) == 0:
            return
        else:
            raise Exception("instances of EmptyDict can't be modified")


class PyListMixin(Bindable, collections.MutableSequence):
    def __getitem__(self, index):
        return self.binder.get_value()[index]
    
    def __len__(self):
        return len(self.binder.get_value())
    
    def __setitem__(self, index, item):
        self.binder.perform_change(ReplaceItem(index, item))
    
    def __delitem__(self, index):
        self.binder.perform_change(DeleteItem(index))
    
    def insert(self, index, item):
        self.binder.perform_change(InsertItem(index, item))
    
    __str__ = list_str
    __repr__ = list_str


class MemoryList(Bindable):
    def __init__(self):
        self._list = TList()
    
    def get_value(self):
        # TODO: Add a copy() to TList and use this, which'll be O(1)
        return self._list
    
    def perform_change(self, change):
        if isinstance(change, InsertItem):
            self._list.insert(change.index, change.item)
        elif isinstance(change, ReplaceItem):
            self._list[change.index] = change.item
        elif isinstance(change, DeleteItem):
            del self._list[change.index]
        elif isinstance(change, SetValue):
            self._list[:] = []
            self._list.extend(change.value)
        else:
            raise TypeError("Need a list-related change")


class PyList(MemoryList, PyListMixin):
    pass


class ListenerBindable(Bindable):
    def __init__(self, listener):
        self.listener = listener
    
    def get_value(self):
        raise SyntheticError
    
    def perform_change(self, change):
        if self.listener is not None:
            self.listener(change)


class _ValueUnwrapperValue(Bindable, TObject):
    def __init__(self, controller):
        TObject.__init__(self)
        self.controller = controller
        self._last_value = None
    
    def get_value(self):
        raise SyntheticError
    
    def perform_change(self, change):
        @stm.atomically
        def _():
            if isinstance(change, SetValue):
                if self._last_value is not None:
                    unbind(self.controller.binding, self._last_value)
                self._last_value = change.value
                # FIXME: Need to figure out what type of binding we're supposed
                # to be doing here (strong/weak), maybe allow the
                # ValueUnwrapperController constructor to specify, or just do
                # as-is for now, I dunno...
                bind(change.value, self.controller.binding)
            elif isinstance(change, LostValue):
                if self._last_value is not None:
                    unbind(self.controller.binding, self._last_value)
                self._last_value = None
            else:
                raise TypeError("Need a SetValue, not %r" % change)


class ValueUnwrapperController(object):
    def __init__(self):
        self.value = _ValueUnwrapperValue(self)
        self.binding = SyntheticBindable()


class _DictControllerKey(PyValueMixin, Bindable, TObject):
    def __init__(self, controller, key):
        TObject.__init__(self)
        self.controller = controller
        self._key = key
    
    def get_value(self):
        return self._key
    
    def perform_change(self, change):
        @stm.atomically
        def _():
            self._key = change.value
            if not self.controller.dict.binder.is_synthetic:
                c = SetValue(self.controller.value.get_value())
                # Notify, not perform; we don't want the value itself trying
                # to perform the change, which would cause it to try to replace
                # the value in the dict, which is pointless.
                self.controller.value.binder.notify_change(c)


class _DictControllerValue(PyValueMixin, Bindable):
    def __init__(self, controller):
        self.controller = controller
    
    def get_value(self):
        # Will throw SyntheticError if it's synthetic, which we'll propagate
        d = self.controller.dict.binder.get_value()
        if self.controller.key._key in d:
            return d[self.controller.key._key]
        else:
            return self.controller._sentinel
    
    def perform_change(self, change):
        if self.controller.dict.binder.is_synthetic:
            # Dict is synthetic, nothing to change
            return
        if change.value == self.controller._sentinel:
            c = DeleteKey(self.controller.key._key)
        else:
            c = ModifyKey(self.controller.key._key, change.value)
        # Just notify the dict of the change.
        self.controller.dict.binder.notify_change(c)


class _DictControllerDict(Bindable):
    def __init__(self, controller):
        self.controller = controller
    
    def get_value(self):
        raise SyntheticError
    
    def perform_change(self, change):
        if isinstance(change, LostValue):
            # Becoming synthetic. Tell the value that we're becoming synthetic,
            # and we're done.
            self.controller.value.binder.notify_change(LostValue())
        elif isinstance(change, (ModifyKey, DeleteKey)):
            if change.key != self.controller.key._key:
                # Change to a key that we're not looking at; do nothing
                return
            # Change to the key we're looking at
            if isinstance(change, ModifyKey):
                # Key has a value or changed values; notify the value of our
                # change
                self.controller.value.binder.notify_change(SetValue(change.value))
            else:
                # Key was deleted; set to the sentinel
                self.controller.value.binder.notify_change(SetValue(self.controller._sentinel))
        elif isinstance(change, SetValue):
            # Figure out what our own key is in the given dict, and notify the
            # value of it. Note that this could result in double notification
            # if we keep getting set to a new dict with our value the same both
            # times, but I don't think double notifications should be a problem
            # for now.
            # TODO: Split this into multiple statements, this line is way too
            # long
            self.controller.value.binder.notify_change(SetValue(change.value.get(self.controller.key._key, self.controller._sentinel)))
        else:
            raise TypeError


class DictController(object):
    # key is the key to start out with. sentinel is the value to expose (and to
    # watch for) when the key doesn't exist (or to delete the key). I may add
    # support later for not having a sentinel, i.e. setting to a nonexistent
    # key either fails validation or automatically creates it, probably by
    # default the former with an option to use the latter. 
    def __init__(self, key, sentinel=NoValue()):
        self.dict = _DictControllerDict(self)
        self.value = _DictControllerValue(self)
        self.key = _DictControllerKey(self, key)
        self._sentinel = sentinel


def value_for_dict_key(d, key, sentinel=NoValue()):
    controller = DictController(key, sentinel)
    s_bind_s(controller.dict, d)
    return controller.value


def value_for_weak_dict_key(d, key, sentinel=NoValue()):
    controller = DictController(key, sentinel)
    w_bind_s(controller.dict, d)
    return controller.value


class _ListTranslatorList(MemoryList):
    def __init__(self, from_function):
        MemoryList.__init__(self)
        self.from_function = from_function
    
    def perform_change(self, change):
        @stm.atomically
        def _():
            if isinstance(change, InsertItem):
                MemoryList.perform_change(self, change)
                translated_change = InsertItem(change.index, self.from_function(change.item))
                MemoryList.perform_change(self.other, translated_change)
                self.other.binder.notify_change(translated_change)
            elif isinstance(change, ReplaceItem):
                MemoryList.perform_change(self, change)
                translated_change = ReplaceItem(change.index, self.from_function(change.item))
                MemoryList.perform_change(self.other, translated_change)
                self.other.binder.notify_change(translated_change)
            elif isinstance(change, DeleteItem):
                MemoryList.perform_change(self, change)
                MemoryList.perform_change(self.other, change)
                self.other.binder.notify_change(change)
            elif isinstance(change, SetValue):
                # TODO: Really ought to split this out for other widgets too lazy
                # to do any special processing to make use of
                for n in reversed(range(len(self.get_value()))):
                    self.perform_change(DeleteItem(n))
                for i, v in enumerate(change.value):
                    self.perform_change(InsertItem(i, v))
            else:
                raise TypeError


class ListTranslator(object):
    def __init__(self, a_to_b, b_to_a):
        self.a = _ListTranslatorList(a_to_b)
        self.b = _ListTranslatorList(b_to_a)
        self.a.other = self.b
        self.b.other = self.a


class _BinaryViewerParam(MemoryValue):
    def __init__(self, viewer, value):
        MemoryValue.__init__(self, value)
        self.viewer = viewer
    
    def perform_change(self, change):
        @stm.atomically
        def _():
            MemoryValue.perform_change(self, change)
            self.viewer.recalculate()


class _BinaryViewerOutput(MemoryValue):
    def __init__(self, viewer):
        MemoryValue.__init__(self, None)
        self.viewer = viewer
    
    def perform_change(self, change):
        raise Exception("Can't modify the output of a binary viewer")


class BinaryViewer(object):
    def __init__(self, function, a, b):
        self.function = function
        self.a = _BinaryViewerParam(self, a)
        self.b = _BinaryViewerParam(self, b)
        self.output = _BinaryViewerOutput(self)
        self.recalculate()
    
    def recalculate(self):
        @stm.atomically
        def _():
            a_value, b_value = self.a.value, self.b.value
            result = self.function(a_value, b_value)
            MemoryValue.perform_change(self.output, SetValue(result))
            self.output.binder.notify_change(SetValue(result))


class _DelayModel(PyValueMixin, SyntheticBindable):
    def __init__(self, controller):
        self.controller = controller
    
    def perform_change(self, change):
        if self.controller.synchronized.get():
            self.controller.view.binder.notify_change(change)


class _DelayView(PyValueMixin, SyntheticBindable):
    def __init__(self, controller):
        self.controller = controller
    
    def perform_change(self, change):
        self.controller.synchronized.binder.perform_change(SetValue(False))


class DelayController(object):
    def __init__(self):
        self.model = _DelayModel(self)
        self.view = _DelayView(self)
        self.synchronized = MemoryValue(True)
    
    def save(self):
        if self.synchronized.get():
            return
        else:
            @stm.atomically
            def _():
                self.model.binder.perform_change(SetValue(self.view.binder.get_value()))
                self.synchronized.binder.perform_change(SetValue(True))
    
    def cancel(self):
        if self.synchronized.get():
            return
        else:
            @stm.atomically
            def _():
                self.view.binder.perform_change(SetValue(self.model.binder.get_value()))
                self.synchronized.binder.perform_change(SetValue(True))


def key_bind(s, s_key, m, m_key, s_weak=True, m_weak=False):
    @stm.atomically
    def _():
        s_controller = DictController(s_key)
        m_controller = DictController(m_key)
        bind(m_controller.dict, m, s_weak, m_weak)
        bind(s_controller.dict, s, m_weak, s_weak)
        bind(s_controller.value, m_controller.value, s_weak, m_weak)

def v_key_bind(s, m, m_key, s_weak=True, m_weak=False):
    @stm.atomically
    def _():
        m_controller = DictController(m_key)
        bind(m_controller.dict, m, s_weak, m_weak)
        bind(s, m_controller.value, s_weak, m_weak)


def key_bind_v(s, s_key, m, s_weak=True, m_weak=False):
    @stm.atomically
    def _():
        s_controller = DictController(s_key)
        bind(s_controller.dict, s, m_weak, s_weak)
        bind(s_controller.value, m, s_weak, m_weak)











































    
    
